@@ -24,15 +24,41 @@
 #   \item{references}{An optional @numeric @vector specifying which samples
 #     should be as reference samples for estimating the model parameters.
 #     If @NULL, all samples are used.}
-#   \item{...}{Arguments passed to calmateByTotalAndFracB or calmateByThetaAB.}
+#   \item{flavor}{A @character string specifying which flavor of the
+#     CalMaTe algorithm to use for fitting the model.
+#     See @see "fitCalMaTeInternal" for details.}
+#   \item{...}{Additional arguments passed to @see "calmateByTotalAndFracB".}
 # }
 #
 # \section{Fields and Methods}{
 #  @allmethods "public"  
 # }
 # 
-# \details{
-#   ...
+# \section{Reference samples}{
+#  In order to estimate the calibration parameters, the model assumes
+#  that, for any given SNP, there are a majority of samples that are
+#  diploid at that SNP.  Note that it does not have to be the same set
+#  of samples for all SNPs.
+#
+#  By using argument \code{references}, it is possible so specify which
+#  samples should be used when estimating the calibration parameters.
+#  This is useful when for instance there are several tumor samples with
+#  unknown properties as well as a set of normal samples that can be
+#  assumed to be diploid.
+#
+#  Theoretical, a minimum of three reference samples are needed in order
+#  for the model to be identifiable.  If less, an error is thrown.
+#  However, in practice more reference samples should be used, that is,
+#  in the order of at least 6-10 reference samples with a diverse set
+#  of genotypes.
+# }
+#
+# \section{Flavors}{
+#   For backward compatibility, we try to keep all major versions of
+#   the CalMaTe algorithm available.  Older versions can be used by
+#   specifying argument \code{flavor}.
+#   For more information about the different flavors, 
+#   see @see "fitCalMaTeInternal".
 # }
 #
 # \examples{\dontrun{
@@ -45,12 +71,15 @@
 #
 # \seealso{
 #   Low-level versions of the CalMaTe method is available
-#   via the @see "calmateByThetaAB.array" and 
-#   @see "calmateByTotalAndFracB.array" methods.
+#   via the @see "calmateByThetaAB" and 
+#   @see "calmateByTotalAndFracB" methods.
+#
+#  For further information on the internal fit functions, see
+#  @see "fitCalMaTeInternal".
 # }
 #
 #*/###########################################################################
-setConstructorS3("CalMaTeCalibration", function(data=NULL, tags="*", references=NULL, ...) {
+setConstructorS3("CalMaTeCalibration", function(data=NULL, tags="*", references=NULL, flavor=c("v2", "v1"), ...) {
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Validate arguments
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -104,22 +133,33 @@ setConstructorS3("CalMaTeCalibration", function(data=NULL, tags="*", references=
     references <- unique(references);
     references <- sort(references);
 
-    if (length(references) < 6) {
-      throw("Argument 'references' specifies too few (<6) reference samples: ", length(references));
+    if (length(references) < 3) {
+      throw("Argument 'references' specifies too less than three reference samples: ", length(references));
     }
   }
 
+  # Argument 'flavor':
+  flavor <- match.arg(flavor);
 
-  # Arguments '...':
-  args <- list(...);
-  if (length(args) > 0) {
-    argsStr <- paste(names(args), collapse=", ");
-    throw("Unknown arguments: ", argsStr);
+  # Arguments '...'; optional arguments to calmateByTotalAndFracB()
+  extraArgs <- list(...);
+  if (length(extraArgs) > 0) {
+    keys <- names(extraArgs);
+    if (is.null(keys)) {
+      throw("Optional arguments to CalMaTeCalibration passed via '...' must be named.");
+    }
+
+    nok <- which(nchar(keys) == 0);
+    if (length(nok) > 0) {
+      throw("All arguments to CalMaTeCalibration passed via '...' must be named.");
+    }
   }
 
-  this <- extend(Object(...), "CalMaTeCalibration",
+  this <- extend(Object(), "CalMaTeCalibration",
     .data = data,
-    .references = references
+    .references = references,
+    .flavor = flavor,
+    .extraArgs = extraArgs
   );
 
   setTags(this, tags);
@@ -143,12 +183,19 @@ setMethodS3("as.character", "CalMaTeCalibration", function(x, ...) {
   }
   refs <- getReferences(this);
   nbrOfFiles <- nbrOfFiles(this);
-  nbrOfRefs <- length(refs);
-  if (nbrOfRefs == 0) 
-    nbrOfRefs <- nbrOfFiles;
   s <- c(s, sprintf("Number of arrays: %d", nbrOfFiles));
-  s <- c(s, sprintf("Number of references: %d (%.2f%%)", nbrOfRefs, 100*nbrOfRefs/nbrOfFiles));
+  nbrOfRefs <- length(refs);
+  if (nbrOfRefs == 0) {
+    s <- c(s, "Number of references: <all arrays> (100%)");
+  } else {
+    s <- c(s, sprintf("Number of references: %d (%.2f%%)", nbrOfRefs, 100*nbrOfRefs/nbrOfFiles));
+  }
  
+  params <- getParametersAsString(this);
+  nparams <- length(params);
+  params <- paste(params, collapse=", ");
+  s <- c(s, sprintf("Additional parameters: [%d] %s", nparams, params));
+
   class(s) <- "GenericSummary";
   s;
 }, private=TRUE)
@@ -157,7 +204,7 @@ setMethodS3("as.character", "CalMaTeCalibration", function(x, ...) {
 
 setMethodS3("getAsteriskTags", "CalMaTeCalibration", function(this, collapse=NULL, ...) {
   tags <- "CMTN";
-
+  tags <- c(tags, this$.flavor);
   if (!is.null(collapse)) {
     tags <- paste(tags, collapse=collapse);
   }
@@ -281,6 +328,25 @@ setMethodS3("getReferences", "CalMaTeCalibration", function(this, ...) {
   this$.references;
 })
 
+setMethodS3("getParameters", "CalMaTeCalibration", function(this, ...) {
+  params <- list();
+  params$references <- getReferences(this);
+  params$flavor <- this$.flavor;
+  params <- c(params, this$.extraArgs);
+  params;
+}, protected=TRUE);
+
+
+setMethodS3("getParametersAsString", "CalMaTeCalibration", function(this, ...) {
+  params <- getParameters(this, expand=FALSE);
+  params <- trim(capture.output(str(params)))[-1];
+  params <- gsub("^[$][ ]*", "", params);
+  params <- gsub(" [ ]*", " ", params);
+  params <- gsub("[ ]*:", ":", params);
+  params;
+}, private=TRUE)
+
+
 
 setMethodS3("getOutputDataSets", "CalMaTeCalibration", function(this, ..., verbose=FALSE) {
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -400,33 +466,110 @@ setMethodS3("findUnitsTodo", "CalMaTeCalibration", function(this, ..., verbose=F
 
   verbose && enter(verbose, "Finding units to do");
 
+
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # (a) Check for missing values in total CNs, because then we will 
+  #     check both non-polymorphic loci and SNPs.
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   dsList <- getOutputDataSets(this);
+  dsT <- dsList$total;
+  verbose && print(verbose, dsT);
 
-  # The last data set is always updated last
-  ds <- dsList[[length(dsList)]];
-  verbose && print(verbose, ds);
-
-  # The last file (in lexicographic ordering) is always updated last
-  fullnames <- getFullNames(ds);
+  fullnames <- getFullNames(dsT);
   verbose && str(verbose, fullnames);
 
   o <- order(fullnames, decreasing=TRUE);
   idx <- o[1];
-  df <- getFile(ds, idx);
-  verbose && print(verbose, df);
+  dfT <- getFile(dsT, idx);
+  verbose && print(verbose, dfT);
 
   # Read all values
-  values <- df[,1,drop=TRUE];
-
-  verbose && cat(verbose, "Number of units: ", length(values));
+  values <- dfT[,1,drop=TRUE];
 
+  nbrOfUnits <- length(values);
+  verbose && cat(verbose, "Number of units: ", nbrOfUnits);
 
   # Identify all missing values
-  nok <- is.na(values);
+  nokT <- is.na(values);
+  unitsT <- which(nokT);
+  verbose && printf(verbose, "Number of TCNs missing values: %d (%.2f%%)\n", length(unitsT), 100*length(unitsT)/nbrOfUnits);
+
+
+
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # (b) In case the BAFs were not stored but the TCNs were, which may
+  #     happen during a user interrupt or power failure and because BAFs
+  #     are stored after TCNs, we must make sure to check those as well.
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # The last data set is always updated last
+  dsB <- dsList$fracB;
+  verbose && print(verbose, dsB);
+
+  # The last file (in lexicographic ordering) is always updated last
+  # (This should be the same as above, but just in case...)
+  fullnames <- getFullNames(dsB);
+  verbose && str(verbose, fullnames);
+
+  o <- order(fullnames, decreasing=TRUE);
+  idx <- o[1];
+  dfB <- getFile(dsB, idx);
+  verbose && print(verbose, dfB);
+
+  # Read all values
+  values <- dfB[,1,drop=TRUE];
+
+  # Identify all missing values
+  nokB <- is.na(values);
+  unitsB <- which(nokB);
+  verbose && printf(verbose, "Number of BAFs with missing values: %d (%.2f%%)\n", length(unitsB), 100*length(unitsB)/nbrOfUnits);
+
+
+  # Identify units that *may* be unfitted.
+  nok <- (nokT | nokB);
   units <- which(nok);
 
+
+  # Potentially unfitted units?
+  if (length(units) > 0) {
+    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # (c) Now, for BAFs we will get missing values for all non-polymorphic 
+    #     loci.  The problem is that we don't know which they are (without 
+    #     turning to chip type-specific annotation data files which cannot
+    #     assume exist).  Instead, we will check the corresponding input
+    #     BAF file to see whether those units also have missing values.
+    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    dsList <- getDataSets(this);
+    # The last data set is always updated last
+    dsB <- dsList$fracB;
+    verbose && print(verbose, dsB);
+  
+    idxs <- indexOf(dsB, getFullName(dfB));
+    # Sanity check
+    stopifnot(length(idxs) > 0);
+  
+    idx <- idxs[1];
+    dfB <- getFile(dsB, idx);
+    verbose && print(verbose, dfB);
+  
+    values <- dfB[,1,drop=TRUE];
+  
+    # Identify all missing values
+    nokBin <- is.na(values);
+    unitsB <- which(nokBin);
+    verbose && printf(verbose, "Number of input BAFs with missing values: %d (%.2f%%)\n", length(unitsB), 100*length(unitsB)/nbrOfUnits);
+  
+    # Conclusions
+    nokB <- (nokB & !nokBin);
+    unitsB <- which(nokB);
+    verbose && printf(verbose, "Number of BAFs with missing values in output but not in input: %d (%.2f%%)\n", length(unitsB), 100*length(unitsB)/nbrOfUnits);
+  
+    # Update units that are unfitted.
+    nok <- (nokT | nokB);
+    units <- which(nok);
+  } # if (length(units) > 0)
+
   verbose && printf(verbose, "Number of units to do: %d (%.2f%%)\n", 
-                       length(units), 100*length(units)/length(values));
+                     length(units), 100*length(units)/length(values));
 
   verbose && cat(verbose, "Units to do (with missing values):");
   verbose && str(verbose, units);
@@ -437,7 +580,7 @@ setMethodS3("findUnitsTodo", "CalMaTeCalibration", function(this, ..., verbose=F
 })
 
 
-setMethodS3("process", "CalMaTeCalibration", function(this, units="remaining", references=getReferences(this), ..., force=FALSE, ram=NULL, verbose=FALSE) {
+setMethodS3("process", "CalMaTeCalibration", function(this, units="remaining", force=FALSE, ram=NULL, verbose=FALSE, ...) {
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Validate arguments
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -470,16 +613,37 @@ setMethodS3("process", "CalMaTeCalibration", function(this, units="remaining", r
     on.exit(popState(verbose));
   }
 
+  # Argument '...':
+  userArgs <- list(...);
+  if (length(userArgs) > 0) {
+    if (is.element("references", names(userArgs))) {
+      throw("Argument 'references' must not be passed via process(), but instead to the CalMaTeCalibration constructor.");
+    }
+
+    warning("Do not pass extra arguments via process(), but instead via CalMaTeCalibration: ", paste(names(userArgs), collapse=", "));
+  }
+
+
+
   verbose && enter(verbose, "CalMaTe calibration of ASCNs");
+
+  params <- getParameters(this);
+
   nbrOfFiles <- nbrOfFiles(this);
-  nbrOfRefs <- length(references);
-  if (nbrOfRefs == 0) nbrOfRefs <- nbrOfFiles;
   verbose && cat(verbose, "Number of arrays: ", nbrOfFiles);
+
+  references <- params$references;
+  nbrOfRefs <- length(references);
+  if (nbrOfRefs == 0L) nbrOfRefs <- nbrOfFiles;
   verbose && cat(verbose, "Number of references: ", nbrOfRefs);
 
   verbose && cat(verbose, "Units:");
   verbose && str(verbose, units);
 
+  verbose && cat(verbose, "All parameters:");
+  verbose && str(verbose, params);
+
+
   # Skip already processed units or not?
   if (!force) {
     verbose && enter(verbose, "Skipping already processed units");
@@ -585,8 +749,17 @@ setMethodS3("process", "CalMaTeCalibration", function(this, units="remaining", r
     verbose && exit(verbose);
 
     verbose && enter(verbose, "Calibration");
-    dataN <- calmateByTotalAndFracB(data, references=references, ..., 
-                                                         verbose=verbose);
+    args <- params;
+    # BACKWARD COMPATIBILITY: Override with user arguments. /HB 2012-02-05
+    for (key in names(userArgs)) {
+      args[[key]] <- userArgs[[key]];
+    }
+    args <- c(list(data), args);
+    verbose && cat(verbose, "Arguments passed to calmateByTotalAndFracB():");
+    verbose && str(verbose, args);
+    args$verbose <- verbose;
+
+    dataN <- do.call("calmateByTotalAndFracB", args=args);
 
     fit <- attr(dataN, "modelFit");
     verbose && str(verbose, fit);
@@ -654,6 +827,29 @@ setMethodS3("process", "CalMaTeCalibration", function(this, units="remaining", r
 
 ############################################################################
 # HISTORY:
+# 2012-02-19 [HB]
+# o Made findUnitsTodo() for CalMaTeCalibration smarter. Before it would
+#   detect all non-polymorphic loci as non-fitted.
+# o Added argument 'flavor' to CalMaTeCalibration, which now also add
+#   the "flavor" to the default set of tags.
+# 2012-02-06 [HB]
+# o Now as.character() also reports any optional parameters.
+# o Added getParametersAsString().
+# 2012-02-05 [HB]
+# o DEPRECATED: Now argument 'references' of process() is obsolete and
+#   gives an error.  Specify it via CalMaTeCalibration() instead.
+# o Added getParameters() for CalMaTeCalibration.
+# 2011-07-15 [HB]
+# o DOCUMENTATION: Added a section 'Reference samples' to the help
+#   of CalMaTeCalibration.
+# o DEPRECATED: Argument 'references' of process() of CalMaTeCalibration
+#   is deprecated.  Instead, pass it to CalMaTeCalibration().
+# o CLEANUP: Tidied up the validation of argument 'references' and
+#   improved the corresponding error messages.
+# 2011-07-12 [MO]
+# o Check that the number of references has to be at least 3.
+#   If no references are given, the total number of files has to be
+#   at least 3. (Used to be 6).
 # 2011-03-12
 # o BUG FIX: After recent update, allocateOutputDataSets() would only
 #   work for existing data sets, not to create new ones.
